/*
Copyright (c) 2009-2013, INRIA, Universite de Nancy 2 and Universidade
Federal do Rio Grande do Norte.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   * Neither the name of the Universite de Nancy 2 or the Universidade Federal
     do Rio Grande do Norte nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.

THIS SOFTWARE IS PROVIDED BY INRIA, Universite de Nancy 2 and
Universidade Federal do Rio Grande do Norte ''AS IS'' AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INRIA, Université de Nancy 2 and
Universidade Federal do Rio Grande do Norte BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

/*
  --------------------------------------------------------------
  Literal management Module
  --------------------------------------------------------------
*/

#include "config.h"

#include "DAG.h"
#include "general.h"
#include "stack.h"

#include "literal.h"

Tvar var_max = 0;
Tvar var_available = 0;
static unsigned var_alloc = 0;
static TDAG *var2DAG = NULL;
static Tvar *DAG2var = NULL;

bool * bool_required = NULL;
#ifdef POLARITY_FILTER
/**
   \brief counter for number of occurrences of literals */
static unsigned * bool_counter = NULL;
#endif

/* #define DEBUG_BOOL */

/*--------------------------------------------------------------*/

static void
literal_DAG_hook_resize(unsigned old_alloc, unsigned new_alloc)
{
  unsigned i;
  MY_REALLOC(DAG2var, new_alloc * sizeof(Tvar));
  for (i = old_alloc; i < new_alloc; i++)
    DAG2var[i] = 0;
}

/*--------------------------------------------------------------*/

Tlit
DAG_is_lit(TDAG DAG)
{
  return DAG2var[DAG] << 1;
}

/*--------------------------------------------------------------*/

Tlit
DAG_to_lit(TDAG DAG)
{
  unsigned positive = 1;
  while (DAG_symb(DAG) == CONNECTOR_NOT)
    {
      DAG = DAG_arg0(DAG);
      positive = !positive;
    }
  if (!DAG2var[DAG])
    {
      var_max++;
      if (var_alloc == var_max)
	{
	  unsigned i;
	  var_alloc *= 2;
	  MY_REALLOC(var2DAG, var_alloc * sizeof(TDAG));
	  MY_REALLOC(bool_required, 2 * var_alloc * sizeof(bool));
#ifdef POLARITY_FILTER
	  MY_REALLOC(bool_counter, 2 * var_alloc * sizeof(unsigned));
#endif
	  for (i = var_max; i < var_alloc; i++)
	    {
	      var2DAG[i] = 0;
	      bool_required[i << 1] = false;
	      bool_required[(i << 1) | 1] = false;
#ifdef POLARITY_FILTER
	      bool_counter[i << 1]= 0;
	      bool_counter[(i << 1) | 1]= 0;	      
#endif
	    }
	}
      DAG2var[DAG] = var_max;
      var2DAG[var_max] = DAG_dup(DAG);
      SAT_var_new_id(var_max);
#ifndef POLARITY_FILTER
      bool_required[var_max << 1] = bool_required[(var_max << 1) | 1] =
        !boolean_connector(DAG_symb(DAG)) && DAG_arity(DAG);
#endif
#ifdef DEBUG_BOOL
      my_DAG_message("DAG variable: %D, %d\n", DAG, var_max);
#endif
    }
  return lit_make(DAG2var[DAG], positive);
}

/*--------------------------------------------------------------*/

TDAG
lit_to_DAG(Tlit lit)
{
  return var2DAG[lit_var(lit)];
}

/*--------------------------------------------------------------*/

TDAG
DAG_to_var(TDAG DAG)
{
  return DAG2var[DAG];
}

/*--------------------------------------------------------------*/

TDAG
var_to_DAG(Tvar var)
{
  assert(var <= var_max);
  return var2DAG[var];
}


/*--------------------------------------------------------------*/

#ifdef POLARITY_FILTER
void
bool_lit_inc(Tlit lit)
{
  if (boolean_connector(DAG_symb(var_to_DAG(lit_var(lit)))) ||
      !DAG_arity(var_to_DAG(lit_var(lit))))
    return;
  my_DAG_message("%d %D\n", lit_var(lit));
  assert(lit_var(lit) <= var_max);
  bool_counter[lit]++;
  bool_required[lit] = true;
}

/*--------------------------------------------------------------*/

void
bool_lit_dec(Tlit lit)
{
  if (boolean_connector(DAG_symb(var_to_DAG(lit_var(lit)))) ||
      !DAG_arity(var_to_DAG(lit_var(lit))))
    return;
  DAG_print(lit_var(lit));
  assert(bool_counter[lit]);
  bool_counter[lit]--;
  if (!bool_counter[lit])
    bool_required[lit] = false;
}
#endif

/*--------------------------------------------------------------*/

void
literal_init(void)
{
  assert(!var2DAG);
  assert(!DAG2var);
  assert(!bool_required);
#ifdef POLARITY_FILTER
  assert(!bool_counter);
#endif
  var_alloc = 64;
  MY_MALLOC(var2DAG, var_alloc * sizeof(TDAG));
  memset(var2DAG, 0, var_alloc * sizeof(TDAG));
  MY_MALLOC(bool_required, 2 * var_alloc * sizeof(bool));
  memset(bool_required, 0, 2 * var_alloc * sizeof(bool));
#ifdef POLARITY_FILTER
  MY_MALLOC(bool_counter, 2 * var_alloc * sizeof(unsigned));
  memset(bool_counter, 0, 2 * var_alloc * sizeof(unsigned));
#endif
  var2DAG[0] = DAG_NULL;
  var_max = 0;
  DAG_set_hook_resize(literal_DAG_hook_resize);
}

/*--------------------------------------------------------------*/

void
literal_reset(void)
{
  unsigned i;
  assert(var2DAG);
  assert(bool_required);
#ifdef POLARITY_FILTER
  assert(bool_counter);
#endif
  for (i = 1; i <= var_max; i++)
    {
      DAG2var[var2DAG[i]] = 0;
      DAG_free(var2DAG[i]);
      var2DAG[i] = 0;
      bool_required[i << 1] = false;
      bool_required[(i << 1) | 1] = false;
#ifdef POLARITY_FILTER
      bool_counter[i << 1]= 0;
      bool_counter[(i << 1) | 1]= 0;	      
#endif
    }
  var_max = 0;
}

/*--------------------------------------------------------------*/

void
literal_done(void)
{
  unsigned i;
  assert(var2DAG);
  for (i = 1; i <= var_max; i++)
    DAG_free(var2DAG[i]);
  free(var2DAG);
  free(bool_required);
#ifdef POLARITY_FILTER
  free(bool_counter);
#endif
}
