/*
Copyright (c) 2009-2013, INRIA, Universite de Nancy 2 and Universidade
Federal do Rio Grande do Norte.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   * Neither the name of the Universite de Nancy 2 or the Universidade Federal
     do Rio Grande do Norte nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.

THIS SOFTWARE IS PROVIDED BY INRIA, Universite de Nancy 2 and
Universidade Federal do Rio Grande do Norte ''AS IS'' AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INRIA, Université de Nancy 2 and
Universidade Federal do Rio Grande do Norte BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

*/

/* DC+DD number representation and manipulation */

#ifndef __NUMBER_H_
#define __NUMBER_H_

#include <limits.h>
#include <string.h>

#include "DAG.h"

#include <gmp.h>

typedef struct TSnumber{
  mpq_t num;
}*Tnumber;

/* DD module initialization: creates useful constants */
extern void    number_init(void);
/* DD module is closed, constants are not available anymore */
extern void    number_done(void);

/* DD+DC some useful constants */
Tnumber number_zero;
Tnumber number_one;
Tnumber number_not_zero;

/* DD numbers are references to objects - Objects must be created with
   number_new() before any use. Once they are no longer useful, they
   shall be disposed with number_free() */

Tnumber number_new(void);
Tnumber number_dup(Tnumber n);
Tnumber number_set(Tnumber dest, const int num);
Tnumber number_cpy(Tnumber dest, const Tnumber src);
void    number_free(Tnumber n);

/* DD arithmetic operations - note that the first two return their
   result, in addition to storing it in the object referenced by
   the first argument;
   this is not the most efficient, but calls may be composed */
Tnumber number_add(Tnumber res, const Tnumber n1, const Tnumber n2);
Tnumber number_sub(Tnumber res, const Tnumber n1, const Tnumber n2);
Tnumber number_mul(Tnumber res, const Tnumber b1, const Tnumber n2);
Tnumber number_div(Tnumber res, const Tnumber b1, const Tnumber n2);

/* DD number_neg modifies its argument */
Tnumber number_neg(Tnumber n);

/* DC changes n to its absolute value */
Tnumber number_abs(Tnumber n);

/* DD conversion functions */

void   number_from_DAG(Tnumber dest, const TDAG DAG);
TDAG   number_to_DAG(const Tnumber n, const Tsort sort);

Tnumber number_from_double(Tnumber dest, const double d);

/* DD + DC comparison functions */

int     number_less(const Tnumber n1, const Tnumber n2);
#define number_equal(n1, n2) (mpq_cmp((n1)->num, (n2)->num) == 0)
int     number_greater(const Tnumber n1, const Tnumber n2);

#define number_leq(n1, n2) (!number_greater((n1), (n2)))
#define number_geq(n1, n2) (!number_less((n1), (n2)))

#define number_min(n1, n2) (number_less(n1, n2) ? n1 : n2)
#define number_max(n1, n2) (number_less(n1, n2) ? n2 : n1)


/* DC The following conversion functions should be only used for debugging.
   If they are really necessary they should be reimplemented properly (if possible) */

#define number_to_double(n) (mpq_get_d(n->num))
char buffer[1000];
#define number_to_str(n) (mpq_get_str(buffer, 10, n->num))

#endif /* __NUMBER_H_*/
